/*
	Поддержка плагина:

	Dev-cs: https://dev-cs.ru/members/1657/
	Telegram: WellAsGood
*/

#include <amxmodx>
#include <reapi>
#include <hamsandwich>

#define CVARS_DESCRIPTION(%0) fmt("%L", LANG_SERVER, %0)

#define IsValidPlayer(%0) (%0 > 0 && %0 <= MaxClients)

#define AUTO_CONFIG //Авто-создание конфига с кварами по пути: '../amxmodx/configs/plugins/'.

#define ARRAYS_BUFFER 64 //Размер для массивов, при необходимости увеличить (например, если длинные названия звуков или моделей)

new const PLUGIN_VER[] = "1.0.1";

new const SETTINGS_FILE_NAME[] = "air_strikes.ini";

const TASK_ID_PLAYER_THINK = 945123;
const TASK_ID_FIX_VELOCITY = 639642;
const TASK_ID_ACTION_END = 386945;

enum any:CVAR_DATA
{
	ACCESS[64],
	Float:DAMAGE_PRE,
	Float:DAMAGE_POST,
	VELOCITY_TO_THE_TARGET,
	Float:VELOCITY_UP,
	Float:DISTANCE,
	STRIKES_PRE,
	Float:STRIKES_PRE_TIME,
	Float:TIME_TO_AIR_STRIKES,
	STRIKES_POST,
	Float:STRIKES_POST_TIME,
	USE_ON_YOUR_TEAM,
	IMMORTALITY_IN_THE_FALL,
	Float:IMMORTALITY_TIME
};

new eCvarData[CVAR_DATA], Float:SaveVelocity[MAX_PLAYERS+1][3], SaveTargetID[MAX_PLAYERS+1];
new Array:SaveModels, Array:SaveSounds, bool:AlreadyUsed[MAX_PLAYERS+1], SaveFlags, CheckDamage[MAX_PLAYERS+1];

public plugin_precache()
{
	register_plugin("Air strikes", PLUGIN_VER, "wellasgood");

	SaveModels = ArrayCreate(ARRAYS_BUFFER);
	SaveSounds = ArrayCreate(ARRAYS_BUFFER);

	new Path[MAX_RESOURCE_PATH_LENGTH];
	get_localinfo("amxx_configsdir", Path, charsmax(Path));

	formatex(Path, charsmax(Path), "%s/%s", Path, SETTINGS_FILE_NAME);

	if(!file_exists(Path))
	{
		set_fail_state("Нету файла '%s'!", Path);
	}

	new Data[ARRAYS_BUFFER], f = fopen(Path, "rt"), Section;

	while(!feof(f))
	{
		fgets(f, Data, charsmax(Data));
		trim(Data);

		if(!Data[0] || Data[0] == ';') continue;

		if(Data[0] == '[')
		{
			if(containi(Data[1], "модели") == 0)
			{
				Section = 1;
			}
			else if(containi(Data[1], "звуки") == 0)
			{
				Section = 2;
			}

			continue;
		}

		switch(Section)
		{
			case 1:
			{
				ArrayPushString(SaveModels, Data);
				precache_model(Data);
			}
			case 2:
			{
				ArrayPushString(SaveSounds, Data);
				precache_sound(Data);
			}
		}
	}

	fclose(f);
}

public plugin_init()
{
	register_dictionary("air_strikes.txt")

	RegisterHookChain(RG_CBasePlayer_UseEmpty, "@UseEmpty_Post", true);
	RegisterHookChain(RG_CSGameRules_CleanUpMap, "@CleanUpMap_Post", true);
	RegisterHookChain(RG_CBasePlayer_TakeDamage, "@TakeDamage_Pre", false);

	CreateCvars();

	#if defined AUTO_CONFIG
	AutoExecConfig(true, "Air_strikes");
	#endif
}

public OnConfigsExecuted()
{
	SaveFlags = read_flags(eCvarData[ACCESS]);
	register_cvar("air_strikes_version", PLUGIN_VER, FCVAR_SERVER|FCVAR_SPONLY|FCVAR_UNLOGGED);
}

public client_connect(id)
{
	AlreadyUsed[id] = false;
	CheckDamage[id] = 0;
}

public client_disconnected(id)
{
	remove_task(id);

	remove_task(id + TASK_ID_PLAYER_THINK);
	remove_task(id + TASK_ID_FIX_VELOCITY);
	remove_task(id + TASK_ID_ACTION_END);
}

@TakeDamage_Pre(const this, pevInflictor, pevAttacker, Float:flDamage, bitsDamageType)
{
	if(!eCvarData[IMMORTALITY_IN_THE_FALL]) return HC_CONTINUE;

	if(this == CheckDamage[this] && is_user_alive(this))
	{
		SetHookChainReturn(ATYPE_INTEGER, 0);
		return HC_SUPERCEDE;
	}

	return HC_CONTINUE;
}

@CleanUpMap_Post()
{
	new PlayersID[MAX_PLAYERS], Num;
	get_players(PlayersID, Num, "h");

	for(new i; i < Num; i++)
	{
		remove_task(PlayersID[i]);

		remove_task(PlayersID[i] + TASK_ID_PLAYER_THINK);
		remove_task(PlayersID[i] + TASK_ID_FIX_VELOCITY);
		remove_task(PlayersID[i] + TASK_ID_ACTION_END);

		AlreadyUsed[PlayersID[i]] = false;
		CheckDamage[PlayersID[i]] = 0;
	}
}

@UseEmpty_Post(id)
{
	if(!(get_user_flags(id) & SaveFlags == SaveFlags)) return;
	if(AlreadyUsed[id]) return;

	new target, body;
	get_user_aiming(id, target, body, 9999);

	if(!IsValidPlayer(target)) return;

	if(!eCvarData[USE_ON_YOUR_TEAM])
	{
		if(get_member(id, m_iTeam) == get_member(target, m_iTeam)) return;
	}

	AlreadyUsed[id] = true;

	set_entvar(target, var_flags, get_entvar(target, var_flags) | FL_FROZEN);

	SaveTargetID[id] = target;

	velocity_by_aim(id, eCvarData[VELOCITY_TO_THE_TARGET], SaveVelocity[id]);

	new Buffer[ARRAYS_BUFFER];
	ArrayGetString(SaveSounds, 0, Buffer, charsmax(Buffer));

	rh_emit_sound2(id, 0, CHAN_STATIC, Buffer, VOL_NORM, ATTN_NORM, 0, PITCH_NORM);

	set_task(0.1, "@PlayerThink", id + TASK_ID_PLAYER_THINK, _, _, "b");
}

@PlayerThink(pId)
{
	new id = pId - TASK_ID_PLAYER_THINK;

	if(!is_user_alive(id) || !is_user_alive(SaveTargetID[id]))
	{
		remove_task(pId);
		AlreadyUsed[id] = false;

		return;
	}

	new Float:Origin[2][3];
	get_entvar(id, var_origin, Origin[0]);
	get_entvar(SaveTargetID[id], var_origin, Origin[1]);

	new Float:Distance = get_distance_f(Origin[0], Origin[1]);

	if(Distance <= eCvarData[DISTANCE])
	{
		remove_task(pId);

		new array[1];
		array[0] = 1;

		new Float:Velocity[3];
		set_entvar(id, var_velocity, Velocity);

		set_task(eCvarData[STRIKES_PRE_TIME], "@StrikeHandler", id, array, sizeof array, "a", eCvarData[STRIKES_PRE]);
		set_task((eCvarData[STRIKES_PRE_TIME] * eCvarData[STRIKES_PRE]) + 0.2, "@NextAction", id);
	}
	else
	{
		set_entvar(id, var_velocity, SaveVelocity[id]);
	}
}

@NextAction(id)
{
	if(!is_user_alive(id) || !is_user_alive(SaveTargetID[id]))
	{
		@ActionEnd(id + TASK_ID_ACTION_END);
		AlreadyUsed[id] = false;

		return;
	}

	new Buffer[ARRAYS_BUFFER];
	ArrayGetString(SaveSounds, 1, Buffer, charsmax(Buffer));

	rh_emit_sound2(id, 0, CHAN_STATIC, Buffer, VOL_NORM, ATTN_NORM, 0, PITCH_NORM);

	set_entvar(SaveTargetID[id], var_flags, get_entvar(SaveTargetID[id], var_flags) & ~FL_FROZEN);

	set_task(0.1, "@FixVelocity", id + TASK_ID_FIX_VELOCITY, _, _, "b");
	set_task(eCvarData[TIME_TO_AIR_STRIKES], "@MainAction", id);
	set_task(eCvarData[TIME_TO_AIR_STRIKES] + eCvarData[STRIKES_POST_TIME], "@FrozenOn", id);
}

@FrozenOn(id)
{
	if(!is_user_alive(SaveTargetID[id])) return;
	set_entvar(SaveTargetID[id], var_flags, get_entvar(SaveTargetID[id], var_flags) | FL_FROZEN);
}

@FixVelocity(pId)
{
	new id = pId - TASK_ID_FIX_VELOCITY;

	if(!is_user_alive(id) || !is_user_alive(SaveTargetID[id]))
	{
		remove_task(pId);
		remove_task(id + TASK_ID_ACTION_END);

		@ActionEnd(id + TASK_ID_ACTION_END);
		AlreadyUsed[id] = false;

		return;
	}

	new Float:Velocity[3];
	Velocity[2] += eCvarData[VELOCITY_UP];

	set_entvar(id, var_velocity, Velocity);
	set_entvar(SaveTargetID[id], var_velocity, Velocity);
}

@MainAction(id)
{
	if(!is_user_alive(id) || !is_user_alive(SaveTargetID[id]))
	{
		remove_task(id + TASK_ID_FIX_VELOCITY);
		remove_task(id + TASK_ID_ACTION_END);

		@ActionEnd(id + TASK_ID_ACTION_END);
		AlreadyUsed[id] = false;

		return;
	}

	new array[1];
	array[0] = 2;

	set_task(eCvarData[STRIKES_POST_TIME], "@StrikeHandler", id, array, sizeof array, "a", eCvarData[STRIKES_POST]);
	set_task((eCvarData[STRIKES_POST_TIME] * eCvarData[STRIKES_POST]) + 0.2, "@TaskVelocityFixOff", id + TASK_ID_FIX_VELOCITY);
	set_task((eCvarData[STRIKES_POST_TIME] * eCvarData[STRIKES_POST]) + 1.0, "@ActionEnd", id + TASK_ID_ACTION_END);
}

@StrikeHandler(num[], id)
{
	if(!is_user_alive(id) || !is_user_alive(SaveTargetID[id]))
	{
		remove_task(id + TASK_ID_FIX_VELOCITY);
		remove_task(id + TASK_ID_ACTION_END);

		AlreadyUsed[id] = false;

		@ActionEnd(id + TASK_ID_ACTION_END);

		return;
	}

	new value = num[0];
	new random_anim = random_num(0, 1);

	new Buffer[ARRAYS_BUFFER];
	ArrayGetString(SaveModels, (random_anim == 1 ? (get_member(id, m_iTeam) == 2 ? 1 : 2) : 0), Buffer, charsmax(Buffer));

	set_entvar(id, var_viewmodel, Buffer);

	PlayerAnimation(id, random_anim == 1 ? random_num(1, 7) : 1);

	new Float:Health = get_entvar(SaveTargetID[id], var_health);

	if(Health <= (value == 1 ? eCvarData[DAMAGE_PRE] : eCvarData[DAMAGE_POST]))
	{
		ExecuteHamB(Ham_Killed, SaveTargetID[id], id, 0);
	}
	else
	{
		set_entvar(SaveTargetID[id], var_health, Health - (value == 1 ? eCvarData[DAMAGE_PRE] : eCvarData[DAMAGE_POST]));
	}

	Buffer[0] = EOS;
	ArrayGetString(SaveSounds, random_num(2, 4), Buffer, charsmax(Buffer));

	rh_emit_sound2(id, 0, CHAN_STATIC, Buffer, VOL_NORM, ATTN_NORM, 0, PITCH_HIGH);
}

@TaskVelocityFixOff(pId)
{
	new id = pId - TASK_ID_FIX_VELOCITY;

	if(eCvarData[IMMORTALITY_IN_THE_FALL] > 0) CheckDamage[id] = id;
	if(eCvarData[IMMORTALITY_IN_THE_FALL] == 2) CheckDamage[SaveTargetID[id]] = SaveTargetID[id];

	remove_task(pId);
}

@ActionEnd(pId)
{
	new id = pId - TASK_ID_ACTION_END;

	AlreadyUsed[id] = false;

	if(is_user_alive(SaveTargetID[id]))
	{
		set_entvar(SaveTargetID[id], var_flags, get_entvar(SaveTargetID[id], var_flags) & ~FL_FROZEN);
	}

	if(is_user_alive(id))
	{
		new item = rg_get_player_item(id, "weapon_knife", KNIFE_SLOT);
		if(item != 0) rg_switch_weapon(id, item);
	}

	set_task(eCvarData[IMMORTALITY_TIME], "@CheckDamageOff", id);
}

@CheckDamageOff(id)
{
	CheckDamage[id] = 0;
	CheckDamage[SaveTargetID[id]] = 0;
}

stock rg_get_player_item(const id, const classname[], const InventorySlotType:slot = NONE_SLOT)
{
	new item = get_member(id, m_rgpPlayerItems, slot);

	while(!is_nullent(item))
	{
		if(FClassnameIs(item, classname)) return item;
		item = get_member(item, m_pNext);
	}

	return 0;
}

static stock PlayerAnimation(const id, const Animation)
{
	set_entvar(id, var_weaponanim, Animation);

	message_begin(MSG_ONE_UNRELIABLE, SVC_WEAPONANIM, .player = id);
	{
		write_byte(Animation);
		write_byte(0);
	}

	message_end();
}

CreateCvars()
{
	bind_pcvar_string(create_cvar(
		.name = "as_access",
		.string = "d",
		.description = CVARS_DESCRIPTION("DESC_ACCESS")),
		eCvarData[ACCESS],
		charsmax(eCvarData[ACCESS])
	);

	bind_pcvar_float(create_cvar(
		.name = "as_damage_pre",
		.string = "2.0",
		.description = CVARS_DESCRIPTION("DESC_DAMAGE_PRE")),
		eCvarData[DAMAGE_PRE]
	);

	bind_pcvar_float(create_cvar(
		.name = "as_damage_post",
		.string = "4.0",
		.description = CVARS_DESCRIPTION("DESC_DAMAGE_POST")),
		eCvarData[DAMAGE_POST]
	);

	bind_pcvar_num(create_cvar(
		.name = "as_velocity_to_the_target",
		.string = "600",
		.description = CVARS_DESCRIPTION("DESC_VELOCITY_TO_THE_TARGET")),
		eCvarData[VELOCITY_TO_THE_TARGET]
	);

	bind_pcvar_float(create_cvar(
		.name = "as_velocity_up",
		.string = "300.0",
		.description = CVARS_DESCRIPTION("DESC_VELOCITY_UP")),
		eCvarData[VELOCITY_UP]
	);

	bind_pcvar_float(create_cvar(
		.name = "as_distance",
		.string = "70.0",
		.description = CVARS_DESCRIPTION("DESC_DISTANCE")),
		eCvarData[DISTANCE]
	);

	bind_pcvar_num(create_cvar(
		.name = "as_strikes_pre",
		.string = "3",
		.description = CVARS_DESCRIPTION("DESC_STRIKES_PRE")),
		eCvarData[STRIKES_PRE]
	);

	bind_pcvar_float(create_cvar(
		.name = "as_strikes_pre_time",
		.string = "0.3",
		.description = CVARS_DESCRIPTION("DESC_STRIKES_PRE_TIME")),
		eCvarData[STRIKES_PRE_TIME]
	);

	bind_pcvar_float(create_cvar(
		.name = "as_time_to_air_strikes",
		.string = "0.5",
		.description = CVARS_DESCRIPTION("DESC_TIME_TO_AIR_STRIKES")),
		eCvarData[TIME_TO_AIR_STRIKES]
	);

	bind_pcvar_num(create_cvar(
		.name = "as_strikes_post",
		.string = "10",
		.description = CVARS_DESCRIPTION("DESC_STRIKES_POST")),
		eCvarData[STRIKES_POST]
	);

	bind_pcvar_float(create_cvar(
		.name = "as_strikes_post_time",
		.string = "0.5",
		.description = CVARS_DESCRIPTION("DESC_STRIKES_POST_TIME")),
		eCvarData[STRIKES_POST_TIME]
	);

	bind_pcvar_num(create_cvar(
		.name = "as_use_on_your_team",
		.string = "1",
		.description = CVARS_DESCRIPTION("DESC_USE_ON_YOUR_TEAM"),
		.has_min = true,
		.min_val = 0.0,
		.has_max = true,
		.max_val = 1.0),
		eCvarData[USE_ON_YOUR_TEAM]
	);

	bind_pcvar_num(create_cvar(
		.name = "as_immortality_in_the_fall",
		.string = "1",
		.description = CVARS_DESCRIPTION("DESC_IMMORTALITY_IN_THE_FALL"),
		.has_min = true,
		.min_val = 0.0,
		.has_max = true,
		.max_val = 2.0),
		eCvarData[IMMORTALITY_IN_THE_FALL]
	);

	bind_pcvar_float(create_cvar(
		.name = "as_immortality_time",
		.string = "3.0",
		.description = CVARS_DESCRIPTION("DESC_IMMORTALITY_TIME")),
		eCvarData[IMMORTALITY_TIME]
	);
}